import { Component, EventEmitter, Input, OnInit, Output, SimpleChanges, ViewChild } from '@angular/core';
import { ActivatedRoute, Params } from '@angular/router';
import { Observable, Subject, from, mergeMap, of, switchMap, takeUntil, tap } from 'rxjs';
import { ComponentHostDirective } from '../directives/component-host.directive';
import { ServiceComponentType, servicesComponentFactory } from '../services-factory';
import { Service, ServiceTypes } from '../components';

@Component({
  selector: 'app-services',
  standalone: true,
  imports: [ComponentHostDirective],
  templateUrl: './services.component.html',
  styleUrls: ['./services.component.scss'],
})
export class ServicesComponent implements OnInit {
  @Input() id?: number;
  @Output() valueChange: EventEmitter<Service> = new EventEmitter<Service>();
  componentList: Service[] = [{
    id: 0,
    title: "11111",
    description: "111",
    dataModel: "ss",
    dataSoure: "sss"
  },
  {
    id: 1,
    title: "222",
    description: "222",
    dataModel: "ss",
    dataSoure: "sss"
  },
  {
    id: 2,
    title: "3333",
    description: "33333",
    dataModel: "ss",
    dataSoure: "sss"
  }]

  @ViewChild(ComponentHostDirective, { static: true }) componentHost!: ComponentHostDirective;

  constructor(private readonly route: ActivatedRoute) { }

  destroy: Subject<void> = new Subject<void>();

  ngOnInit(): void { // subscribing to the stream when component has been created
    // this.getOne(this.id!).subscribe(result => {
    //   this.loadComponent(result)
    // })
    this.route.params
      .pipe(
        mergeMap(({ id }: Params) => this.getOne(id! ? id : this.id)),
        tap((service: Service) => this.loadComponent(service)),
        takeUntil(this.destroy.asObservable()),
      )
      .subscribe();






  }

  ngOnChanges(changes: SimpleChanges): void {
    console.log(changes)
    if (changes) {
      this.id = changes['id'].currentValue
      this.getOne(this.id!).subscribe(result => {
        this.loadComponent(result)
      })
    }

    //Called before any other lifecycle hook. Use it to inject dependencies, but avoid any serious work here.
    //Add '${implements OnChanges}' to the class.

  }

  loadComponent(service: Service): void {
    console.log("service", service)
    const { viewContainerRef }: ComponentHostDirective = this.componentHost;
    viewContainerRef.clear();

    const { instance } = viewContainerRef.createComponent<ServiceComponentType>(
      servicesComponentFactory[service.id as ServiceTypes]
    );
    instance.model = service.dataModel;
    this.valueChange.emit(service);

  }

  ngOnDestroy(): void {
    this.destroy.next();
  }

  getOne(num: number): Observable<Service> {
    console.log("num", num)
    let componentFilter = this.componentList.filter(e =>
      e.id == num)[0];
    return of(componentFilter)
  }
}
